/*--------------------------------------------------------------------------+
$Id: codetemplates.xml 1 2011-01-01 00:00:01Z hoelzl $
|                                                                          |
| Copyright 2015 ForTISS GmbH                     |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.tooling.base.ui.editor;

import static org.eclipse.gef.SnapToGrid.PROPERTY_GRID_ENABLED;
import static org.eclipse.gef.SnapToGrid.PROPERTY_GRID_ORIGIN;
import static org.eclipse.gef.SnapToGrid.PROPERTY_GRID_SPACING;
import static org.eclipse.gef.SnapToGrid.PROPERTY_GRID_VISIBLE;
import static org.fortiss.tooling.base.layout.DefaultLayoutConstants.DEFAULT_GRID_ORIGIN_X;
import static org.fortiss.tooling.base.layout.DefaultLayoutConstants.DEFAULT_GRID_ORIGIN_Y;
import static org.fortiss.tooling.base.layout.DefaultLayoutConstants.DEFAULT_GRID_SIZE;
import static org.fortiss.tooling.kernel.ui.util.EObjectSelectionUtils.getFirstElement;

import java.util.ArrayList;
import java.util.List;

import org.conqat.ide.commons.ui.selection.SelectionUtils;
import org.eclipse.draw2d.geometry.Dimension;
import org.eclipse.draw2d.geometry.Point;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.gef.EditDomain;
import org.eclipse.gef.EditPart;
import org.eclipse.gef.GraphicalViewer;
import org.eclipse.gef.SnapToGeometry;
import org.eclipse.gef.commands.CommandStack;
import org.eclipse.gef.ui.actions.ActionRegistry;
import org.eclipse.gef.ui.actions.UpdateAction;
import org.eclipse.gef.ui.parts.ScrollingGraphicalViewer;
import org.eclipse.jface.action.IAction;
import org.eclipse.jface.viewers.IPostSelectionProvider;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.ISelectionChangedListener;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.ui.IEditorInput;
import org.eclipse.ui.IEditorSite;
import org.eclipse.ui.ISelectionListener;
import org.eclipse.ui.IWorkbenchPart;
import org.eclipse.ui.PartInitException;
import org.fortiss.tooling.base.model.element.IHierarchicElement;
import org.fortiss.tooling.base.ui.editpart.ExtendedLayerRootEditPart;
import org.fortiss.tooling.kernel.ui.extension.base.EObjectActionBase;
import org.fortiss.tooling.kernel.ui.extension.data.ContextMenuContextProvider;
import org.fortiss.tooling.kernel.ui.service.IActionService;
import org.fortiss.tooling.kernel.ui.service.IAllocationEditPartFactoryService;

/**
 * <p>
 * GEF editor base implementation for linking two GEF based graphical viewer editors.
 * </p>
 * <p>
 * It provides {@link EditDomain} and thus a {@link CommandStack}.
 * </p>
 * 
 * @author eder
 * @author $Author: hoelzl $
 * @version $Rev: 18709 $
 * @ConQAT.Rating RED Hash:
 */
public class AllocationDiagramEditorBase<T extends EObject> extends GEFEditorBase<T> implements
		IPostSelectionProvider, ContextMenuContextProvider {

	/** Actions which need update upon selection changes. */
	private final List<String> selectionActions = new ArrayList<String>();

	/** Stores the viewer. */
	protected GraphicalViewer viewer = new ScrollingGraphicalViewer();

	/** A container for editor actions. */
	private final ActionRegistry actionRegistry = new ActionRegistry();

	/** {@inheritDoc} */
	@Override
	public void init(IEditorSite site, IEditorInput input) throws PartInitException {
		super.init(site, input);

		getSite().getWorkbenchWindow().getSelectionService()
				.addSelectionListener(selectionListener);

		// Install adapter to top-most object to watch for addition of IConnections.
		// if(numberOfConnectionsToBeCreatedUntilCreationHintIsHidden > 0) {
		// EObject model = getEditedObject();
		// EObject obj = model;
		// while(obj != null) {
		// obj = obj.eContainer();
		// }
		//
		// if(!model.eAdapters().contains(addConnectionAdapter)) {
		// model.eAdapters().add(addConnectionAdapter);
		// }
		// }
		viewer.setKeyHandler(new DiagramKeyHandler(viewer));
	}

	/** {@inheritDoc} */
	@Override
	public void createPartControl(Composite parent) {
		viewer.setRootEditPart(new ExtendedLayerRootEditPart());
		viewer.createControl(parent);

		getEditDomain().addViewer(viewer);

		viewer.setEditPartFactory(IAllocationEditPartFactoryService.INSTANCE);
		viewer.setContents(getViewerContentObject());

		getSite().setSelectionProvider(this);

		viewer.getControl().setBackground(parent.getBackground());

		viewer.setProperty(SnapToGeometry.PROPERTY_SNAP_ENABLED, true);
		viewer.setProperty(PROPERTY_GRID_ORIGIN, new Point(DEFAULT_GRID_ORIGIN_X,
				DEFAULT_GRID_ORIGIN_Y));

		Dimension gridSpacing = new Dimension(DEFAULT_GRID_SIZE, DEFAULT_GRID_SIZE);
		viewer.setProperty(PROPERTY_GRID_SPACING, gridSpacing);
		viewer.setProperty(PROPERTY_GRID_ENABLED, true);
		viewer.setProperty(PROPERTY_GRID_VISIBLE, true);

	}

	/** Selection listener used to update selection actions. */
	private final ISelectionListener selectionListener = new ISelectionListener() {

		/** {@inheritDoc} */
		@Override
		public void selectionChanged(IWorkbenchPart part, ISelection selection) {
			EditPart ep = SelectionUtils.checkAndPickFirstSafe(selection, EditPart.class);
			if(ep != null) {
				Control control = ep.getViewer().getControl();
				Object model = ep.getModel();

				String hint = "";
				// if(numberOfConnectionsToBeCreatedUntilCreationHintIsHidden > 0) {
				// if(model instanceof IConnector) {
				// hint =
				// "Link two connectors by pressing ALT and dragging from source to target.";
				// } else
				if(model instanceof IHierarchicElement) {
					hint = "Create a connection by pressing ALT and dragging.";
				}
				// }
				control.setToolTipText(hint);
			}
			updateActions(selectionActions);
		}
	};

	/** Updates a list of actions. */
	@SuppressWarnings({"rawtypes", "unchecked"})
	private void updateActions(List<String> actionIds) {
		EObject element = getSelectedModelElement();
		for(String id : actionIds) {
			IAction action = actionRegistry.getAction(id);
			if(action instanceof UpdateAction) {
				((UpdateAction)action).update();
			} else if(action instanceof EObjectActionBase) {
				((EObjectActionBase)action).refresh(element);
			}
		}
		IActionService.INSTANCE.refresh();
	}

	/**
	 * Returns the root object used as content root. Sub-classes may override to
	 * provide a different content root object. The default implementation uses
	 * {@link GEFEditorBase#getEditedObject()}
	 */
	protected Object getViewerContentObject() {
		return getEditedObject();
	}

	/** {@inheritDoc} */
	@Override
	public void addSelectionChangedListener(ISelectionChangedListener listener) {
		// TODO Auto-generated method stub

	}

	/** {@inheritDoc} */
	@Override
	public ISelection getSelection() {
		return viewer.getSelection();
	}

	/** {@inheritDoc} */
	@Override
	public void removeSelectionChangedListener(ISelectionChangedListener listener) {
		// TODO Auto-generated method stub

	}

	/** {@inheritDoc} */
	@Override
	public void setSelection(ISelection selection) {
		// TODO Auto-generated method stub

	}

	/** {@inheritDoc} */
	@Override
	public EObject getSelectedModelElement() {
		if(getSite().getSelectionProvider() != null) {
			return getFirstElement(getSite().getSelectionProvider().getSelection());
		}
		return null;
	}

	/** {@inheritDoc} */
	@Override
	public List<EObject> getSelectedModelElementList() {
		// TODO Auto-generated method stub
		return null;
	}

	/** {@inheritDoc} */
	@Override
	public void addPostSelectionChangedListener(ISelectionChangedListener listener) {
		// TODO Auto-generated method stub

	}

	/** {@inheritDoc} */
	@Override
	public void removePostSelectionChangedListener(ISelectionChangedListener listener) {
		// TODO Auto-generated method stub

	}

}
