/*--------------------------------------------------------------------------+
$Id: codetemplates.xml 1 2011-01-01 00:00:01Z hoelzl $
|                                                                          |
| Copyright 2015 ForTISS GmbH                     |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.tooling.base.ui.editpart;

import static org.eclipse.gef.LayerConstants.CONNECTION_LAYER;

import org.eclipse.draw2d.BendpointConnectionRouter;
import org.eclipse.draw2d.ColorConstants;
import org.eclipse.draw2d.ConnectionAnchor;
import org.eclipse.draw2d.ConnectionLayer;
import org.eclipse.draw2d.ConnectionRouter;
import org.eclipse.draw2d.FreeformLayer;
import org.eclipse.draw2d.IFigure;
import org.eclipse.draw2d.XYLayout;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.gef.ConnectionEditPart;
import org.eclipse.gef.EditPart;
import org.eclipse.gef.NodeEditPart;
import org.eclipse.gef.Request;
import org.eclipse.swt.SWT;
import org.fortiss.tooling.base.ui.layout.IDiagramLayoutConfiguration;
import org.fortiss.tooling.base.ui.layout.RectangleGridLayoutDiagramConfiguration;

/**
 * The base class for allocation diagrams.
 * 
 * @param <T>
 *            the type of the model element shown.
 * 
 * @author eder
 * @author $Author: hoelzl $
 * @version $Rev: 18709 $
 * @ConQAT.Rating RED Hash:
 */
public abstract class AllocationDiagramEditPartBase<T extends EObject> extends
		GraphicalEditPartBase<T> implements NodeEditPart/*
														 * , IHierarchicElementBehavior<S,
														 * V>
														 */{

	/** Stores the diagram layout configuration. */
	protected final IDiagramLayoutConfiguration layoutConfiguration;

	/** Stores the connection router. */
	private final ConnectionRouter connectionRouter;

	// /** Displayed model children of allocation element 1 */
	// private List<Object> modelChildren1;
	// /** Displayed model children of allocation element 2 */
	// private List<Object> modelChildren2;

	// /** Class of S */
	// private Class<S> clazzS;
	//
	// /** Class of V */
	// private Class<V> clazzV;

	/** Constructor. */
	protected AllocationDiagramEditPartBase(T modelElement) {
		super(modelElement);
		this.layoutConfiguration = RectangleGridLayoutDiagramConfiguration.INSTANCE;
		this.connectionRouter = new BendpointConnectionRouter();
		// this.clazzS = clazzS;
		// this.clazzV = clazzV;
		// modelChildren1 = new ArrayList<>();
		// modelChildren2 = new ArrayList<>();
		// modelChildren1.addAll(getInitialModelChildren1());
		// modelChildren2.addAll(getInitialModelChildren2());
	}

	// /** {@inheritDoc} */
	// @Override
	// final List<?> getModelChildren() {
	// List<Object> result = new ArrayList<>();
	// result.addAll(modelChildren1);
	// result.addAll(modelChildren2);
	// return result;
	// }

	// /** Returns all the initially displayed elements of allocation element 1 */
	// abstract protected List<?> getInitialModelChildren1();
	//
	// /** Returns all the initially displayed elements of allocation element 2 */
	// abstract protected List<?> getInitialModelChildren2();

	/** {@inheritDoc} */
	@Override
	public ConnectionAnchor getSourceConnectionAnchor(ConnectionEditPart connection) {
		return null;
	}

	/** {@inheritDoc} */
	@Override
	public ConnectionAnchor getTargetConnectionAnchor(ConnectionEditPart connection) {
		return null;
	}

	/** {@inheritDoc} */
	@Override
	public ConnectionAnchor getSourceConnectionAnchor(Request request) {
		return null;
	}

	/** {@inheritDoc} */
	@Override
	public ConnectionAnchor getTargetConnectionAnchor(Request request) {
		return null;
	}

	/** {@inheritDoc} */
	@Override
	protected void createEditPolicies() {
		// nothing to do yet
	}

	/** {@inheritDoc} */
	@Override
	protected void refreshVisuals() {
		super.refreshVisuals();

		refreshSourceConnections();
		refreshTargetConnections();
		refreshChildren();

		for(Object o : getChildren()) {
			((EditPart)o).refresh();
		}

		for(Object editPart : getSourceConnections()) {
			((EditPart)editPart).refresh();
		}
		for(Object editPart : getTargetConnections()) {
			((EditPart)editPart).refresh();
		}
	}

	/** {@inheritDoc} */
	@Override
	public IDiagramLayoutConfiguration getLayoutConfiguration() {
		return layoutConfiguration;
	}

	/** {@inheritDoc} */
	@Override
	protected IFigure createFigure() {
		setConnectionRouter();

		FreeformLayer f = new FreeformLayer() {
			/** {@inheritDoc} */
			@Override
			protected boolean useLocalCoordinates() {
				// TODO Auto-generated method stub
				return true;
			}
		};
		f.setOpaque(false);
		f.setLayoutManager(new XYLayout());
		f.setBackgroundColor(ColorConstants.white);
		return f;
	}

	/** Sets anti-aliasing and the connection router on connection layer. */
	private void setConnectionRouter() {
		ConnectionLayer cLayer = (ConnectionLayer)getLayer(CONNECTION_LAYER);
		if(cLayer != null) {
			cLayer.setAntialias(SWT.ON);
			cLayer.setConnectionRouter(connectionRouter);
		}
	}

	// /** {@inheritDoc} */
	// @SuppressWarnings("unchecked")
	// @Override
	// public void performRequest(Request req) {
	// if(req.getType() == RequestConstants.REQ_OPEN) {
	// Set<?> keySet = req.getExtendedData().keySet();
	// Object object = keySet.stream().findFirst().get();
	// if(clazzS.isInstance(object)) {
	// List<Object> subElements1 = (List<Object>)getSubElements1((S)object);
	// if(!subElements1.isEmpty()) {
	// modelChildren1 = subElements1;
	// refresh();
	// }
	// } else if(clazzV.isInstance(object)) {
	// List<Object> subElements2 = (List<Object>)getSubElements2((V)object);
	// if(!subElements2.isEmpty()) {
	// modelChildren2 = subElements2;
	// refresh();
	// }
	// }
	// }
	//
	// super.performRequest(req);
	// }

}
