/*--------------------------------------------------------------------------+
$Id$
|                                                                          |
| Copyright 2011 ForTISS GmbH                                              |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.tooling.base.ui.editpart.command;

import static org.fortiss.tooling.base.ui.utils.DragAndDropBaseUtils.setConnectionDragContextTarget;

import org.eclipse.draw2d.geometry.Point;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.gef.EditPart;
import org.eclipse.gef.commands.Command;
import org.fortiss.tooling.base.dnd.ConnectionDragContext;
import org.fortiss.tooling.base.ui.editpart.policy.DirectConnectionAndReconnectionEditPolicy;
import org.fortiss.tooling.base.utils.LayoutModelElementFactory;
import org.fortiss.tooling.kernel.service.IConnectionCompositorService;

/**
 * This command is used by {@link DirectConnectionAndReconnectionEditPolicy} to
 * communicate with {@link IConnectionCompositorService} for creating new
 * connections.
 * 
 * @author hoelzl
 * @author $Author$
 * @version $Rev$
 * @ConQAT.Rating YELLOW Hash: F18FCFEB90D25F3323A6FB5B6656EC31
 */
public class ConnectCommand extends Command {

	/** Stores DropContext information. */
	protected final ConnectionDragContext context;

	/** Constructor. */
	public ConnectCommand(ConnectionDragContext context) {
		this.context = context;
	}

	/** Sets the target edit part and location. */
	public void setTarget(EditPart targetObject, Point targetLocation) {
		Object modelElement = targetObject.getModel();
		if(modelElement instanceof EObject) {
			LayoutModelElementFactory.createPoint(targetLocation.x(), targetLocation.y(),
					targetLocation.toString() + "_element");
			setConnectionDragContextTarget(context, targetObject, targetLocation);
		}
	}

	/** {@inheritDoc} */
	@Override
	public boolean canExecute() {
		if(context == null || context.getSource() == null || context.getTarget() == null) {
			return false;
		}
		EObject commonParent = getCommonParent();
		if(commonParent == null) {
			return false;
		}
		return IConnectionCompositorService.INSTANCE.canConnect(context.getSource(),
				context.getTarget(), getCommonParent(), context);
	}

	/** {@inheritDoc} */
	@Override
	public void execute() {
		IConnectionCompositorService.INSTANCE.connect(context.getSource(), context.getTarget(),
				getCommonParent(), context);
	}

	/** Returns the common parent. */
	private EObject getCommonParent() {
		return findCommonParent(context.getSourceParent(), context.getTargetParent());
	}

	/** Finds the common parent of two elements. */
	private EObject findCommonParent(EObject sourceParent, EObject targetParent) {
		if(sourceParent == targetParent) {
			return sourceParent;
		}
		if(sourceParent.eContainer() == targetParent) {
			return targetParent;
		}

		if(sourceParent == targetParent.eContainer())
			return sourceParent;

		return sourceParent.eContainer();
	}
}
