/*--------------------------------------------------------------------------+
$Id$
|                                                                          |
| Copyright 2011 ForTISS GmbH                     |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.tooling.base.compose;

import org.fortiss.tooling.base.model.element.IConnector;
import org.fortiss.tooling.base.model.element.IHierarchicElement;
import org.fortiss.tooling.kernel.extension.data.IConnectionCompositionContext;
import org.fortiss.tooling.kernel.extension.data.ITopLevelElement;
import org.fortiss.tooling.kernel.service.IPersistencyService;

/**
 * Base implementation for supporting direct connection of two {@link IHierarchicElement}s.
 * 
 * @author hoelzl
 * @author $Author$
 * @version $Rev$
 * @ConQAT.Rating YELLOW Hash: 09D5C8054487253D2E3C784F9609FAB8
 */
public abstract class HierarchicElementConnectionCompositorBase<HE extends IHierarchicElement, S extends IHierarchicElement, T extends IHierarchicElement>
		extends ConnectionCompositorBase<HE, S, T> {

	/** Creates the application specific source {@link IConnector} model element. */
	protected abstract IConnector createEntryConnector();

	/** Creates the application specific target {@link IConnector} model element. */
	protected abstract IConnector createExitConnector();

	/** {@inheritDoc} */
	@Override
	public boolean canConnect(S source, T target, HE parent, IConnectionCompositionContext context) {
		return true;
	}

	/** {@inheritDoc} */
	@Override
	public boolean connect(S source, T target, HE parent, IConnectionCompositionContext context) {
		IConnector sourceConnector = createEntryConnector();
		avoidDuplicateConnectorName(source, sourceConnector);
		IConnector targetConnector = createExitConnector();
		avoidDuplicateConnectorName(target, targetConnector);
		ITopLevelElement topElement = IPersistencyService.INSTANCE.getTopLevelElementFor(parent);
		if(topElement != null) {
			topElement.prepareIDs(sourceConnector);
			topElement.prepareIDs(targetConnector);
		}
		createNewSourceDropContext(source, context, sourceConnector);
		createNewTargetDropContext(target, context, targetConnector);
		createConnection(targetConnector, sourceConnector, parent, topElement);
		return true;
	}
}
