/*--------------------------------------------------------------------------+
$Id$
|                                                                          |
| Copyright 2012 ForTISS GmbH                     |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.tooling.graphicsGL.ui.objects;

import static org.fortiss.tooling.graphicsGL.ui.util.GLPrimitives.restoreMatrix;
import static org.fortiss.tooling.graphicsGL.ui.util.GLPrimitives.rotate;
import static org.fortiss.tooling.graphicsGL.ui.util.GLPrimitives.saveMatrix;

import org.lwjgl.util.vector.Vector3f;

/**
 * Rotates the wrapped object by the given angle at the origin.
 * 
 * @author hoelzl
 * @author $Author$
 * @version $Rev$
 * @ConQAT.Rating GREEN Hash: EC97A6995AB526518E5CA3B33E4BB1EC
 */
public class RotatedObject extends WrappedViewObjectBase {
	/** The rotation angle. */
	private float angle;

	/** The rotation axis. */
	private final Vector3f axis;

	/** Constructor. */
	public RotatedObject(ViewObjectBase wrapped, float angle, Vector3f axis) {
		super(wrapped);
		this.angle = angle;
		this.axis = axis;
	}

	/** {@inheritDoc} */
	@Override
	protected void preRendering() {
		saveMatrix();
		rotate(angle, axis);
	}

	/** {@inheritDoc} */
	@Override
	protected void postRendering() {
		restoreMatrix();
	}
}
