/*--------------------------------------------------------------------------+
$Id: codetemplates.xml 1 2011-01-01 00:00:01Z hoelzl $
|                                                                          |
| Copyright 2015 ForTISS GmbH                     |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.tooling.kernel.ui.internal;

import java.util.List;

import org.eclipse.emf.ecore.EObject;
import org.eclipse.gef.EditPart;
import org.fortiss.tooling.kernel.service.base.EObjectAwareServiceBase;
import org.fortiss.tooling.kernel.ui.ToolingKernelUIActivator;
import org.fortiss.tooling.kernel.ui.extension.IAllocationEditPartFactory;
import org.fortiss.tooling.kernel.ui.service.IAllocationEditPartFactoryService;
import org.fortiss.tooling.kernel.utils.LoggingUtils;

/**
 * This class implements the {@link IAllocationEditPartFactoryService} interface
 * 
 * @author eder
 * @author $Author: hoelzl $
 * @version $Rev: 18709 $
 * @ConQAT.Rating RED Hash:
 */
public class AllocationEditPartFactoryService extends
		EObjectAwareServiceBase<IAllocationEditPartFactory> implements
		IAllocationEditPartFactoryService {

	/** The prototype provider extension point ID. */
	private static final String EXTENSION_POINT_NAME =
			"org.fortiss.tooling.kernel.ui.allocationEditPartFactory";

	/** The prototype provider configuration element name. */
	private static final String CONFIGURATION_ELEMENT_NAME = "allocationEditPartFactory";

	/** The model element handler class attribute name. */
	private static final String HANDLER_CLASS_ATTRIBUTE_NAME = "factory";

	/** {@inheritDoc} */
	@Override
	public EditPart createEditPart(EditPart context, Object model) {
		if(!(model instanceof EObject)) {
			LoggingUtils.error(ToolingKernelUIActivator.getDefault(),
					AllocationEditPartFactoryService.class.getSimpleName() +
							" may only be used for EObjects!");
			return null;
		}
		Class<? extends EObject> clazz = ((EObject)model).getClass();

		List<IAllocationEditPartFactory> factories = getRegisteredHandlers(clazz);
		if(factories == null || factories.isEmpty()) {
			LoggingUtils.error(ToolingKernelUIActivator.getDefault(),
					"No editpart factory registered for " + clazz.getName());
			return null;
		}

		for(IAllocationEditPartFactory factory : factories) {
			EditPart part = factory.createEditPart(context, model);
			if(part != null) {
				return part;
			}
		}

		return null;

	}

	/** {@inheritDoc} */
	@Override
	protected String getExtensionPointName() {
		return EXTENSION_POINT_NAME;
	}

	/** {@inheritDoc} */
	@Override
	protected String getConfigurationElementName() {
		return CONFIGURATION_ELEMENT_NAME;
	}

	/** {@inheritDoc} */
	@Override
	protected String getHandlerClassAttribute() {
		return HANDLER_CLASS_ATTRIBUTE_NAME;
	}

}
