/*--------------------------------------------------------------------------+
$Id$
|                                                                          |
| Copyright 2012 ForTISS GmbH                     |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.tooling.kernel.utils;

import java.util.ArrayList;
import java.util.List;

/**
 * Class containing mixed useful methods.
 * 
 * @author doebber
 * @author $Author$
 * @version $Rev$
 * @ConQAT.Rating GREEN Hash: 0AEE36671E84E87EA4BFDB32EC21DAC4
 */
public class JavaUtils {

	/**
	 * List revert utility method as Collections.reverse() does not seem to have
	 * any effect.
	 */
	@SuppressWarnings({"rawtypes", "unchecked"})
	public static ArrayList reverse(ArrayList list) {
		// Inefficient but solves problems with Collections.reverse().
		ArrayList result = new ArrayList();
		for(int i = list.size() - 1; i >= 0; i--) {
			result.add(list.get(i));
		}
		return result;
	}

	/**
	 * Converts an List of a given type into an List of one of its subtypes.
	 * Utility method to avoid unnecessary casts.
	 * 
	 * @param targetClass
	 *            - the class representing the subtype
	 * @param sourceList
	 *            - the source list containing objects of type S, a supertype of
	 *            T
	 * @return a target List containing elements of type targetClazz
	 */
	@SuppressWarnings("unchecked")
	public static <S, T extends S> List<T> convertList(Class<T> targetClass, List<S> sourceList) {
		return (List<T>)(List<?>)sourceList;
	}
}
