/*-------------------------------------------------------------------------+
|                                                                          |
| Copyright 2005-2011 The ConQAT Project                                   |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+-------------------------------------------------------------------------*/
package org.conqat.ide.commons.ui.dialog;

import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.layout.GridDataFactory;
import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Text;

/**
 * This dialog allows to show a short message plus a long details message that
 * is shown in read-only text box that also supports copy-paste.
 * 
 * @author deissenb
 * @author $Author: deissenb $
 * @version $Rev: 34252 $
 * @levd.rating GREEN Hash: CAD88E34CB20256B436EB817095381D4
 */
public class DetailsDialog extends MessageDialog {

	/** Details text. */
	private final String details;

	/**
	 * Create new dialog with a simple ok button.
	 * 
	 * @param kind
	 *            one of the following values:
	 *            <ul>
	 *            <li><code>MessageDialog.NONE</code> for a dialog with no image
	 *            </li>
	 *            <li><code>MessageDialog.ERROR</code> for a dialog with an
	 *            error image</li>
	 *            <li><code>MessageDialog.INFORMATION</code> for a dialog with
	 *            an information image</li>
	 *            <li><code>MessageDialog.QUESTION </code> for a dialog with a
	 *            question image</li>
	 *            <li><code>MessageDialog.WARNING</code> for a dialog with a
	 *            warning image</li>
	 *            </ul>
	 * @param title 
	 *            dialog title
	 * @param message
	 *            message (short)
	 * @param details
	 *            details (long)
	 */
	public DetailsDialog(int kind, String title, String message, String details) {
		this(kind, title, message, details, new String[] { "Ok" }, 0);
	}

	/**
	 * Create dialog (parameters see
	 * {@link #DetailsDialog(int, String, String, String)}. Additionally one may
	 * specify:
	 * 
	 * @param buttonTexts
	 *            texts of buttons
	 * @param defaultButtonIndex
	 *            index (within <code>buttonTexts</code>) of the default button
	 */
	public DetailsDialog(int kind, String title, String message,
			String details, String[] buttonTexts, int defaultButtonIndex) {
		super(Display.getCurrent().getActiveShell(), title, null, message,
				kind, buttonTexts, defaultButtonIndex);
		this.details = details;
		setShellStyle(getShellStyle() | SWT.RESIZE);
	}

	/** Create text box below message. */
	@Override
	protected Control createCustomArea(Composite parent) {
		Text text = new Text(parent, SWT.BORDER | SWT.READ_ONLY | SWT.MULTI
				| SWT.V_SCROLL | SWT.H_SCROLL);
		text.setText(details);
		GridDataFactory.defaultsFor(text).grab(true, true).applyTo(text);
		return text;
	}

}